﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using GadgeteerManager;
using Windows.Networking;
using Windows.Networking.Sockets;
using Windows.Storage.Streams;
using Windows.UI.Xaml;


namespace CameraControlLibrary
{
    public class RobotControl : IDisposable
    {
        private DataWriter tcpStream;
        private DispatcherTimer tmrPing;
        private StreamSocket socket;
        private string LastCommand;

        public async Task ConnectAsync(string address, string password)
        {
            if (string.IsNullOrWhiteSpace(address))
                throw new ArgumentNullException(address);

            try
            {
                string[] addressPart = address.Split(':');
                socket = new StreamSocket();
                await socket.ConnectAsync(new HostName(addressPart[0]),
                                                                       addressPart[1].ToString(),
                                                                       SocketProtectionLevel.PlainSocket);
                tcpStream = new DataWriter(socket.OutputStream);
                tmrPing = new DispatcherTimer(); // new Timer((stateInfo) => Ping(), null,
                //  (int)TimeSpan.FromSeconds(30).TotalMilliseconds, (int)TimeSpan.FromSeconds(30).TotalMilliseconds);
                tmrPing.Interval = TimeSpan.FromSeconds(30);
                tmrPing.Tick += tmrPing_Tick;
                tmrPing.Start();
            }
            catch
            {
                throw new ConnectionException("Rover not found or connection error");
            }
        }

        void tmrPing_Tick(object sender, object e)
        {
            Ping();
        }

        public void Disconnect()
        {
            tmrPing.Stop();

            tcpStream.Dispose();
            tcpStream = null;
            socket.Dispose();
        }

        public void Ping()
        {
            this.SendCommandAsync(RobotCommands.PING);
        }

        public void MotorOn()
        {
            this.SendCommandAsync(RobotCommands.TURN_ON);
        }

        public void MotorOff()
        {
            this.SendCommandAsync(RobotCommands.TURN_OFF);
        }

        public void ChangeSpeed(int speed)
        {
            var command = string.Format("{0}:{1}", RobotCommands.SET_SPEED, speed);
            this.SendCommandAsync(command);
        }

        public void MoveForeward()
        {
            this.SendCommandAsync(RobotCommands.MOVE_FORWARD);
        }

        public void MoveBackward()
        {
            this.SendCommandAsync(RobotCommands.MOVE_BACKWARD);
        }

        public void TurnLeft()
        {
            this.SendCommandAsync(RobotCommands.TURN_LEFT);
        }

        public void TurnRight()
        {
            this.SendCommandAsync(RobotCommands.TURN_RIGHT);
        }

        public void MoveForewardLeft()
        {
            this.SendCommandAsync(RobotCommands.MOVE_FORWARD + RobotCommands.TURN_LEFT);
        }

        public void MoveForewardRight()
        {
            this.SendCommandAsync(RobotCommands.MOVE_FORWARD + RobotCommands.TURN_RIGHT);
        }

        public void MoveBackwardLeft()
        {
            this.SendCommandAsync(RobotCommands.MOVE_BACKWARD + RobotCommands.TURN_LEFT);
        }

        public void MoveBackwardRight()
        {
            this.SendCommandAsync(RobotCommands.MOVE_BACKWARD + RobotCommands.TURN_RIGHT);
        }

        public void Stop()
        {
            this.SendCommandAsync(RobotCommands.STOP);
        }

        public void Rotate()
        {
            this.SendCommandAsync(RobotCommands.ROTATE);
        }

        public void Dispose()
        {
            this.Disconnect();
        }

        private async Task SendCommandAsync(string command)
        {
            Debug.WriteLine("Sending command: " + command);
            if (tcpStream != null)
            {
                if (command != LastCommand)
                {

                    var buffer = Encoding.UTF8.GetBytes(command);
                    tcpStream.WriteBytes(buffer);

                    try
                    {
                        await tcpStream.StoreAsync();
                    }
                    catch (Exception exc)
                    {
                        Debug.WriteLine(exc.Message);
                        //_view.DisplayErrorMessageFormat("Cannot send data to device.\n{0} - {1}.",
                        //                                exc.Message,
                        //                                SocketError.GetStatus(exc.HResult));
                    }
                    LastCommand = command;
                }

            }
        }
    }
}
