/* Copyright 2012 Marco Minerva, marco.minerva@gmail.com

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using System;
using Microsoft.SPOT;
using System.Xml;
using System.IO;
using System.Collections;

namespace GadgeteerRobot
{
    public static class ConfigurationManager
    {
        private const string APPSETTINGS_SECTION = "appSettings";
        private const string ADD = "add";
        private const string KEY = "key";
        private const string VALUE = "value";

        private static Hashtable appSettings;        

        static ConfigurationManager()
        {
            appSettings = new Hashtable();
        }

        public static string GetAppSetting(string key)
        {
            return GetAppSetting(key, null);
        }

        public static string GetAppSetting(string key, string defaultValue)
        {
            if (!appSettings.Contains(key))
                return defaultValue;

            return (string)appSettings[key];
        }

        public static void SetAppSetting(string key, string value)
        {
            appSettings[key] = value;
        }

        public static void Load(Stream xmlStream)
        {
            appSettings.Clear();
            using (XmlReader reader = XmlReader.Create(xmlStream))
            {
                while (reader.Read())
                {
                    switch (reader.Name)
                    {
                        case APPSETTINGS_SECTION:
                            while (reader.Read())
                            {
                                if (reader.Name == APPSETTINGS_SECTION)
                                    break;

                                if (reader.Name == ADD)
                                {
                                    var key = reader.GetAttribute(KEY);
                                    var value = reader.GetAttribute(VALUE);

                                    //Debug.Print(key + "=" + value);
                                    appSettings.Add(key, value);
                                }
                            }

                            break;
                    }
                }
            }
        }

        public static void Save(Stream xmlStream)
        {
            using (StreamWriter writer = new StreamWriter(xmlStream))
            {
                writer.WriteLine("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
                writer.WriteLine("<configuration>");
                writer.WriteLine("\t<appSettings>");

                foreach (DictionaryEntry item in appSettings)
                {
                    string add = "\t\t<add key=\"" + item.Key + "\" value=\"" + item.Value + "\" />";
                    writer.WriteLine(add);
                }

                writer.WriteLine("\t</appSettings>");
                writer.WriteLine("</configuration>");
            }
        }
    }
}
