/* Copyright 2012 Marco Minerva, marco.minerva@gmail.com

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.
*/

using System;
using Microsoft.SPOT;
using System.Net.Sockets;
using System.Threading;
using System.Net;

namespace NetUtils
{
    public class UdpSocketServer
    {    
        public const int DEFAULT_SERVER_PORT = 8080;

        private int port;   
        private Socket socket;

        public delegate void DataReceivedEventHandler(object sender, UdpDataReceivedEventArgs e);
        public event DataReceivedEventHandler DataReceived;

        public UdpSocketServer()
            : this(DEFAULT_SERVER_PORT)
        { }
        
        public UdpSocketServer(int port)
        {
            this.port = port;
            socket = new Socket(AddressFamily.InterNetwork, SocketType.Dgram, ProtocolType.Udp);
        }

        public void Start()
        {     
            new Thread(StartServerInternal).Start();
        }

        private void StartServerInternal()
        {
            EndPoint endPoint = new IPEndPoint(IPAddress.Any, port);
            socket.Bind(endPoint);      
            
            while (true)
            {
                try
                {
                    if (socket.Poll(-1, SelectMode.SelectRead))
                    {
                        byte[] buffer = new byte[socket.Available];
                        int bytesRead = socket.ReceiveFrom(buffer, ref endPoint);

                        var args = new UdpDataReceivedEventArgs(endPoint, buffer);
                        OnDataReceived(args);

                        if (args.ResponseData != null)
                            socket.SendTo(args.ResponseData, endPoint);
                    }
                    else
                    {
                        Thread.Sleep(10);
                    }
                }
                catch { }
            }
        }

        private void OnDataReceived(UdpDataReceivedEventArgs e)
        {
            if (DataReceived != null)
                DataReceived(this, e);
        }

        public static string BytesToString(byte[] bytes)
        {
            int length = bytes.Length;
            char[] text = new char[length]; 
            for (int i = 0; i < length; i++)
                text[i] = (char)bytes[i]; 
            
            return new string(text);
        }
    }
}
