﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GadgeteerManager
{
    /// <summary>
    /// The <strong>CameraDirection</strong> enum defines the direction to which the camera can be rotated.
    /// </summary>
    public enum CameraDirection
    {
        /// <summary>
        /// Turns the camera on the left.
        /// </summary>
        Left, 
        /// <summary>
        /// Turns the camera on the right.
        /// </summary>
        Right
    }

    /// <summary>
    /// The <strong>ICameraControl</strong> interface defines the commands that control a direction and image stream of a Gadgeteer Camera.
    /// </summary>
    /// <seealso cref="IGadgeteerControl"/>
    public interface ICameraControl : IGadgeteerControl
    {
        /// <summary>
        /// Sends the command to center the camera.
        /// </summary>
        void Center();

        /// <summary>
        /// Sends the command to move the camera of a given step, in the specified direction.
        /// </summary>
        /// <param name="cameraDirection">The direction to which the camera must be rotated.</param>
        /// <param name="step">The step of the rotation, expressed as percentage (0-100).</param>
        void Move(CameraDirection cameraDirection, int step);

        /// <summary>
        /// Gets the image that is captured by the camera.
        /// </summary>
        /// <returns>The byte array that represents the image, in Bitmap format.</returns>
        byte[] GetImage();
    }
}
