﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GadgeteerManager
{
    /// <summary>
    /// The <strong>IGadgeteerControl</strong> interface defines the commands that allows to connect and disconnet to a Gadgeteer device.
    /// </summary>
    public interface IGadgeteerControl
    {
        /// <summary>
        /// Establishes the connection with Gadgeteer device.
        /// </summary>
        /// <param name="address">The address of Gadgeteer device.</param>
        /// <param name="password">The password for the connection, if necessary.</param>
        /// <exception cref="System.ArgumentNullException">The <paramref name="address"/> parameter is null or empty.</exception>
        /// <exception cref="ConnectionException">An error occurred during connection attempt.</exception>
        void Connect(string address, string password);

        /// <summary>
        /// Ends the connection with Gadgeteer device.
        /// </summary>
        void Disconnect();

        /// <summary>
        /// Sends a Keep alive message to Gadgeteer device, in order to mantain the connection open.
        /// </summary>
        void Ping();
    }
}
