﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace GadgeteerManager
{
    /// <summary>
    /// The <strong>IRobotControl</strong> interface defines the commands that can be send to control behaviour and movements of Gadgeteer Robot.
    /// </summary>
    /// <seealso cref="IGadgeteerControl"/>
    public interface IRobotControl : IGadgeteerControl
    {
        /// <summary>
        /// Sends the command to turn on the Gadgeteer Robot, making it avaible to receive other commands.
        /// </summary>
        void MotorOn();

        /// <summary>
        /// Sends the command to turn off the Gadgeteer Robot, disabling all its activities.
        /// </summary>
        void MotorOff();

        /// <summary>
        /// Sends the command to change current speed of Gadgeteer Robot.
        /// </summary>
        /// <param name="speed">The new speed, expressed in percentage. Values can vary from 0 to 100.</param>
        void ChangeSpeed(int speed);

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move forward.
        /// </summary>
        void MoveForeward();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move backward.
        /// </summary>
        void MoveBackward();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot turn left.
        /// </summary>
        void TurnLeft();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot turn right.
        /// </summary>
        void TurnRight();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move forward-left.
        /// </summary>
        void MoveForewardLeft();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move forward-right.
        /// </summary>
        void MoveForewardRight();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move backward-left.
        /// </summary>
        void MoveBackwardLeft();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot move backward-right.
        /// </summary>
        void MoveBackwardRight();

        /// <summary>
        /// Sends the command to make Gadgeteer Robot rotate on itself.
        /// </summary>
        void Rotate();

        /// <summary>
        /// Sends the command to stop Gadgeteer Robots movements.
        /// </summary>
        void Stop();
    }
}
