﻿
Imports KinectManager.Core.Gestures
Imports Microsoft.Kinect
Imports KinectManager.Core.Handlers
Imports KinectManager.Core.EventArgs

Namespace Handlers
    Public Class RightHandStoppedGesture
        Inherits GestureHandlerBase

        Public Property GestureDuration As TimeSpan = New TimeSpan(0, 0, 3)

        Private Property XPrecision As Double
        Private Property YPrecision As Double
        Private Property ZPrecision As Double


        Public Overrides Sub Elaborate(ByRef stopElaborate As Boolean)
            Dim skeletons = GetTrackedSkeletons()
            If skeletons IsNot Nothing AndAlso skeletons.Any() Then
                For Each s In skeletons
                    ElaborateGesture(s)
                Next
            End If
        End Sub

        Private Sub ElaborateGesture(skeleton As Skeleton)
            Dim skeletons = GetLastSkeletonsByTimespanAndTrackid(GestureDuration, skeleton.TrackingId)
            Dim gestureOk = True
            If skeletons.Count > 1 Then
                For index = 0 To skeletons.Count - 2
                    Dim firstPos = skeletons(index).Joints(JointType.HandRight).Position()
                    Dim secondPos = skeletons(index + 1).Joints(JointType.HandRight).Position()
                    If Math.Abs(firstPos.X - secondPos.X) > XPrecision Then
                        gestureOk = False
                        Exit For
                    End If
                    If Math.Abs(firstPos.Y - secondPos.Y) > YPrecision Then
                        gestureOk = False
                        Exit For
                    End If
                    If Math.Abs(firstPos.Z - secondPos.Z) > ZPrecision Then
                        gestureOk = False
                        Exit For
                    End If
                Next
                If gestureOk Then
                    Dim e = New GestureRecognizedEventArgs
                    e.Skeleton = skeleton
                    e.Gesture = New Gesture() With {.Name = "RightHandStopped", .Data = Nothing}
                    OnGestureRecognized(e)
                End If
            End If
        End Sub

        Public Overrides Property Name As String = Me.GetType.Name
    End Class
End Namespace