﻿

Imports System.IO
Imports GadgeteerManager
Imports StateMachineManager.Implementations
Imports KinectManager.Core
Imports Microsoft.Speech.AudioFormat
Imports Microsoft.Kinect
Imports Microsoft.Practices.Unity
Imports System.Threading
Imports Microsoft.Speech.Recognition
Imports Coding4Fun.Kinect.Wpf
Imports System.Reflection
Imports System.Threading.Tasks
Imports System.Configuration

Namespace ViewModels
    Public MustInherit Class GadgeteerViewModelBase
        Inherits KinectViewModelBase

        Public Sub New(container As IUnityContainer)
            MyBase.New(container)
            If IsInDesignMode Then Return
            CreateRobotControl()
            CreateCameraControl()
        End Sub

        Protected Property CameraTask As Task
        Protected Property RobotControl As IRobotControl
        Protected Property CameraControl As ICameraControl

        Protected Overrides Sub Dispose(disposing As Boolean)
            MyBase.Dispose(disposing)
            If disposing Then
                If RobotControl IsNot Nothing Then
                    RobotControl.Disconnect()
                End If
                If CameraControl IsNot Nothing Then
                    CameraControl.Disconnect()
                End If
            End If
        End Sub

#Region "Properties"
        Private _RobotState As RobotState
        Public Property RobotState As RobotState
            Get
                Return Me._RobotState
            End Get
            Set(value As RobotState)
                If Me._RobotState <> value Then
                    Me._RobotState = value
                    OnPropertyChanged("RobotState")
                End If
            End Set
        End Property

        Private _CurrentCameraImage As BitmapSource
        Public Property CurrentCameraImage As BitmapSource
            Get
                Return Me._CurrentCameraImage
            End Get
            Set(value As BitmapSource)
                Me._CurrentCameraImage = value
                OnPropertyChanged("CurrentCameraImage")
            End Set
        End Property

        ReadOnly Property RobotAddress As String
            Get
                Return ConfigurationManager.AppSettings("RobotAddress")
            End Get
        End Property

        ReadOnly Property RobotPassword As String
            Get
                Return ConfigurationManager.AppSettings("RobotPassword")
            End Get
        End Property

        ReadOnly Property CameraAddress As String
            Get
                Return ConfigurationManager.AppSettings("CameraAddress")
            End Get
        End Property

        ReadOnly Property CameraPassword As String
            Get
                Return ConfigurationManager.AppSettings("CameraPassword")
            End Get
        End Property

        Private _IsCameraOn As Boolean
        Public Property IsCameraOn As Boolean
            Get
                Return _IsCameraOn
            End Get
            Set(value As Boolean)
                If value <> _IsCameraOn Then
                    Me._IsCameraOn = value
                    OnPropertyChanged("IsCameraOn")
                End If
            End Set
        End Property
#End Region

#Region "Creazione Robot e Camera control"
        Private Sub CreateRobotControl()
            RobotControl = Container.Resolve(Of IRobotControl)()
            RobotControl.Connect(RobotAddress, CameraAddress)
        End Sub

        Private Sub CreateCameraControl()
            CameraTask = Task.Run(AddressOf CameraTaskHandler, TaskCancellationSource.Token)
            If Container.IsRegistered(GetType(ICameraControl)) Then
                CameraControl = Container.Resolve(Of ICameraControl)()
            End If
        End Sub

        Private Sub CameraTaskHandler()
            While Not TaskCancellationSource.Token.IsCancellationRequested
                Task.Delay(25)
                If CameraControl IsNot Nothing AndAlso IsCameraOn Then
                    Dim imageArray = CameraControl.GetImage()
                    Dim img = CreateImageFromCamera(imageArray)
                    img.Freeze()
                    Me.CurrentCameraImage = img
                Else
                    Me.CurrentCameraImage = Nothing
                End If
            End While
        End Sub

        Private Function CreateImageFromCamera(imageArray As Byte()) As BitmapImage
            Dim retImage As BitmapImage = Nothing
            Try
                retImage = New BitmapImage()
                retImage.BeginInit()
                retImage.StreamSource = New MemoryStream(imageArray)
                retImage.EndInit()
            Catch ex As Exception
                retImage = Nothing
            End Try
            Return retImage
        End Function
#End Region

    End Class
End Namespace
