﻿Imports StateMachineManager.Implementations
Imports KinectManager.Core
Imports Microsoft.Kinect
Imports Microsoft.Practices.Unity
Imports Coding4Fun.Kinect.Wpf


Namespace ViewModels
    Public Class GadgeteerWindowViewModel
        Inherits GadgeteerViewModelBase

        Shared Sub New()
            CreateRobotStateMachine()
        End Sub

        Public Sub New(container As IUnityContainer)
            MyBase.New(container)
            CreateReferenceAreas()
            SpeechRecognitionDictionaty = New Dictionary(Of String, Action) From {{"ELEVATIONUP", Sub() SetElevationAngle(Sensor.ElevationAngle + 5)},
                                                                                  {"ELEVATIONDOWN", Sub() SetElevationAngle(Sensor.ElevationAngle - 5)},
                                                                                  {"ELEVATIONRESET", Sub() SetElevationAngle(0)},
                                                                                  {"MOTORON", Sub() MotorOn()},
                                                                                  {"MOTOROFF", Sub()
                                                                                                   MotorOff()
                                                                                                   StateMachineController.ChangeState(Of GadgeteerWindowViewModel, RobotState, RobotGesture)(Me, RobotGesture.None)
                                                                                               End Sub},
                                                                                  {"CAMERAON", Sub() CameraOn()},
                                                                                  {"CAMERAOFF", Sub() CameraOff()}}
        End Sub

        Protected Const ImageColorWidth As Integer = 640
        Protected Const ImageColorHeight As Integer = 480

#Region "Funzioni di utilità"
        Protected Function GetCurrentJointXPosition(jointType As JointType) As Single
            If Me.CurrentSkeleton IsNot Nothing Then
                If Me.CurrentSkeleton.Joints(jointType).TrackingState <> JointTrackingState.NotTracked Then
                    Return Me.CurrentSkeleton.Joints(jointType).ScaleTo(ImageColorWidth, ImageColorHeight).Position.X
                Else
                    Return Single.NaN
                End If
            End If
            Return Single.NaN
        End Function

        Protected Function GetCurrentJointYPosition(jointType As JointType) As Single
            If Me.CurrentSkeleton IsNot Nothing Then
                If Me.CurrentSkeleton.Joints(jointType).TrackingState <> JointTrackingState.NotTracked Then
                    Return Me.CurrentSkeleton.Joints(jointType).ScaleTo(ImageColorWidth, ImageColorHeight).Position.Y
                Else
                    Return Single.NaN
                End If
            End If
            Return Single.NaN
        End Function
#End Region

#Region "Reference Areas"
        Private Sub CreateReferenceAreas()
            Me.SkeletalTrackingReferenceArea = CreateSkeletalTrackingReferenceArea()
            CreateForwardBackwardReferenceAreas()
            CreateTurnLeftRightReferenceAreas()
            CreateCameraReferenceAreas()
        End Sub

        Private Function CreateSkeletalTrackingReferenceArea() As ReferenceArea
            Dim referenceArea = New ReferenceArea() With {.Name = "SkeletalTracking",
                                                          .X1 = -0.3D, .X2 = 0.3D,
                                                          .Y1 = -1, .Y2 = 1,
                                                          .Type = ReferenceAreaType.SkeletalTrackingArea}
            Return referenceArea
        End Function

        Private Sub CreateForwardBackwardReferenceAreas()
            Dim referenceArea = New ReferenceArea() With {.Name = "Forward",
                                                      .X1 = -0.5D, .X2 = -0.2D,
                                                      .Y1 = 0.1D, .Y2 = 0.4D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.ForwardReferenceArea = referenceArea

            referenceArea = New ReferenceArea() With {.Name = "Backward",
                                                      .X1 = -0.5D, .X2 = -0.2D,
                                                      .Y1 = -0.4D, .Y2 = -0.1D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.BackwardReferenceArea = referenceArea
        End Sub

        Private Sub CreateTurnLeftRightReferenceAreas()
            Dim referenceArea = New ReferenceArea() With {.Name = "TurnLeft",
                                                      .X1 = 0.1D, .X2 = 0.4D,
                                                      .Y1 = -0.35D, .Y2 = -0.05D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.TurnLeftReferenceArea = referenceArea

            referenceArea = New ReferenceArea() With {.Name = "TurnRight",
                                                      .X1 = 0.5D, .X2 = 0.8D,
                                                      .Y1 = -0.35D, .Y2 = -0.05D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.TurnRightReferenceArea = referenceArea
        End Sub

        Private Sub CreateCameraReferenceAreas()
            Dim referenceArea = New ReferenceArea() With {.Name = "CameraRotateCW",
                                                      .X1 = 0.1D, .X2 = 0.4D,
                                                      .Y1 = 0.1D, .Y2 = 0.4D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.CameraRotateCWReferenceArea = referenceArea

            referenceArea = New ReferenceArea() With {.Name = "CameraRotateCCW",
                                                      .X1 = 0.5D, .X2 = 0.8D,
                                                      .Y1 = 0.1D, .Y2 = 0.4D,
                                                      .Type = ReferenceAreaType.GestureArea}
            Me.CameraRotateCCWReferenceArea = referenceArea
        End Sub

#End Region

#Region "Properties"

        Private _SkeletalTrackingReferenceArea As ReferenceArea
        Public Property SkeletalTrackingReferenceArea As ReferenceArea
            Get
                Return Me._SkeletalTrackingReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._SkeletalTrackingReferenceArea = value
                OnPropertyChanged("SkeletalTrackingReferenceArea")
            End Set
        End Property

        Private _ForwardReferenceArea As ReferenceArea
        Public Property ForwardReferenceArea As ReferenceArea
            Get
                Return Me._ForwardReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._ForwardReferenceArea = value
                OnPropertyChanged("ForwardReferenceArea")
            End Set
        End Property

        Private _BackwardReferenceArea As ReferenceArea
        Public Property BackwardReferenceArea As ReferenceArea
            Get
                Return Me._BackwardReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._BackwardReferenceArea = value
                OnPropertyChanged("BackwardReferenceArea")
            End Set
        End Property

        Private _TurnRightReferenceArea As ReferenceArea
        Public Property TurnRightReferenceArea As ReferenceArea
            Get
                Return Me._TurnRightReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._TurnRightReferenceArea = value
                OnPropertyChanged("TurnRightReferenceArea")
            End Set
        End Property

        Private _TurnLeftReferenceArea As ReferenceArea
        Public Property TurnLeftReferenceArea As ReferenceArea
            Get
                Return Me._TurnLeftReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._TurnLeftReferenceArea = value
                OnPropertyChanged("TurnLeftReferenceArea")
            End Set
        End Property

        Private _CameraRotateCWReferenceArea As ReferenceArea
        Public Property CameraRotateCWReferenceArea As ReferenceArea
            Get
                Return Me._CameraRotateCWReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._CameraRotateCWReferenceArea = value
                OnPropertyChanged("CameraRotateCWReferenceArea")
            End Set
        End Property

        Private _CameraRotateCCWReferenceArea As ReferenceArea
        Public Property CameraRotateCCWReferenceArea As ReferenceArea
            Get
                Return Me._CameraRotateCCWReferenceArea
            End Get
            Set(value As ReferenceArea)
                Me._CameraRotateCCWReferenceArea = value
                OnPropertyChanged("CameraRotateCCWReferenceArea")
            End Set
        End Property

        Private _IsForwardReferenceAreaHighlight As Boolean = False
        Public Property IsForwardReferenceAreaHighlight As Boolean
            Get
                Return _IsForwardReferenceAreaHighlight
            End Get
            Set(value As Boolean)
                If value <> _IsForwardReferenceAreaHighlight Then
                    _IsForwardReferenceAreaHighlight = value
                    OnPropertyChanged("IsForwardReferenceAreaHighlight")
                End If
            End Set
        End Property

        Private _IsBackwardReferenceAreaHighlight As Boolean = False
        Public Property IsBackwardReferenceAreaHighlight As Boolean
            Get
                Return _IsBackwardReferenceAreaHighlight
            End Get
            Set(value As Boolean)
                If value <> _IsBackwardReferenceAreaHighlight Then
                    _IsBackwardReferenceAreaHighlight = value
                    OnPropertyChanged("IsBackwardReferenceAreaHighlight")
                End If
            End Set
        End Property

        Private _IsTurnLeftReferenceAreaHighlight As Boolean = False
        Public Property IsTurnLeftReferenceAreaHighlight As Boolean
            Get
                Return _IsTurnLeftReferenceAreaHighlight
            End Get
            Set(value As Boolean)
                If value <> _IsTurnLeftReferenceAreaHighlight Then
                    _IsTurnLeftReferenceAreaHighlight = value
                    OnPropertyChanged("IsTurnLeftReferenceAreaHighlight")
                End If
            End Set
        End Property

        Private _IsTurnRightReferenceAreaHighlight As Boolean = False
        Public Property IsTurnRightReferenceAreaHighlight As Boolean
            Get
                Return _IsTurnRightReferenceAreaHighlight
            End Get
            Set(value As Boolean)
                If value <> _IsTurnRightReferenceAreaHighlight Then
                    _IsTurnRightReferenceAreaHighlight = value
                    OnPropertyChanged("IsTurnRightReferenceAreaHighlight")
                End If
            End Set
        End Property

        Private _IsCameraRotateCWReferenceArea As Boolean = False
        Public Property IsCameraRotateCWReferenceArea As Boolean
            Get
                Return _IsCameraRotateCWReferenceArea
            End Get
            Set(value As Boolean)
                If value <> _IsCameraRotateCWReferenceArea Then
                    _IsCameraRotateCWReferenceArea = value
                    OnPropertyChanged("IsCameraRotateCWReferenceArea")
                End If
            End Set
        End Property

        Private _IsCameraRotateCCWReferenceArea As Boolean = False
        Public Property IsCameraRotateCCWReferenceArea As Boolean
            Get
                Return _IsCameraRotateCCWReferenceArea
            End Get
            Set(value As Boolean)
                If value <> _IsCameraRotateCCWReferenceArea Then
                    _IsCameraRotateCCWReferenceArea = value
                    OnPropertyChanged("IsCameraRotateCCWReferenceArea")
                End If
            End Set
        End Property

#End Region

#Region "Kinect"

#Region "Gestures"
        Protected Overrides Sub SkeletonNotTracked()
            StateMachineController.ChangeState(Of GadgeteerWindowViewModel, RobotState, RobotGesture)(Me, RobotGesture.None)
            Me.IsTurnLeftReferenceAreaHighlight = False
            Me.IsTurnRightReferenceAreaHighlight = False
            Me.IsForwardReferenceAreaHighlight = False
            Me.IsBackwardReferenceAreaHighlight = False
        End Sub

        Protected Overrides Sub ManageGestures()
            Dim handRightGesture = ManageRightHand()
            Dim handLeftGesture = ManageLeftHand()

            Dim gesture = handRightGesture Or handLeftGesture

            StateMachineController.ChangeState(Of GadgeteerWindowViewModel, RobotState, RobotGesture)(Me, gesture)

            If CameraControl IsNot Nothing AndAlso IsCameraOn Then
                ManageCameraGesture()
            End If
        End Sub


        Protected Function ManageRightHand() As RobotGesture
            If CurrentSkeleton IsNot Nothing Then
                Dim skeletonHandRight = CurrentSkeleton.Joints(JointType.HandRight)
                If skeletonHandRight.TrackingState <> JointTrackingState.NotTracked Then
                    Dim handRight = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(skeletonHandRight.Position, Sensor.ColorStream.Format)
                    If TurnLeftReferenceArea.IsInArea(handRight, skeletonHandRight.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsTurnLeftReferenceAreaHighlight = True
                        Me.IsTurnRightReferenceAreaHighlight = False
                        Return RobotGesture.Left
                    End If
                    If TurnRightReferenceArea.IsInArea(handRight, skeletonHandRight.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsTurnLeftReferenceAreaHighlight = False
                        Me.IsTurnRightReferenceAreaHighlight = True
                        Return RobotGesture.Right
                    End If
                End If
            End If
            Me.IsTurnLeftReferenceAreaHighlight = False
            Me.IsTurnRightReferenceAreaHighlight = False
            Return RobotGesture.None
        End Function

        Protected Function ManageLeftHand() As RobotGesture
            If CurrentSkeleton IsNot Nothing Then
                Dim skeletonHandLeft = CurrentSkeleton.Joints(JointType.HandLeft)
                If skeletonHandLeft.TrackingState <> JointTrackingState.NotTracked Then
                    Dim handLeft = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(skeletonHandLeft.Position, Sensor.ColorStream.Format)
                    If ForwardReferenceArea.IsInArea(handLeft, skeletonHandLeft.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsForwardReferenceAreaHighlight = True
                        Me.IsBackwardReferenceAreaHighlight = False
                        Return RobotGesture.Forward
                    End If
                    If BackwardReferenceArea.IsInArea(handLeft, skeletonHandLeft.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsBackwardReferenceAreaHighlight = True
                        Me.IsForwardReferenceAreaHighlight = False
                        Return RobotGesture.Backward
                    End If
                End If
            End If
            Me.IsForwardReferenceAreaHighlight = False
            Me.IsBackwardReferenceAreaHighlight = False
            Return RobotGesture.None
        End Function


        Protected Sub ManageCameraGesture()
            If CurrentSkeleton IsNot Nothing Then
                Dim skeletonHandRight = CurrentSkeleton.Joints(JointType.HandRight)
                If skeletonHandRight.TrackingState <> JointTrackingState.NotTracked Then
                    Dim handRight = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(skeletonHandRight.Position, Sensor.ColorStream.Format)
                    If CameraRotateCWReferenceArea.IsInArea(handRight, skeletonHandRight.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsCameraRotateCWReferenceArea = True
                        Me.IsCameraRotateCCWReferenceArea = False
                        CameraRotate(True)
                    ElseIf CameraRotateCCWReferenceArea.IsInArea(handRight, skeletonHandRight.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                        Me.IsCameraRotateCWReferenceArea = False
                        Me.IsCameraRotateCCWReferenceArea = True
                        CameraRotate(False)
                    Else
                        Me.IsCameraRotateCWReferenceArea = False
                        Me.IsCameraRotateCCWReferenceArea = False
                    End If
                Else
                    Me.IsCameraRotateCWReferenceArea = False
                    Me.IsCameraRotateCCWReferenceArea = False
                End If
            Else
                Me.IsCameraRotateCWReferenceArea = False
                Me.IsCameraRotateCCWReferenceArea = False
            End If
        End Sub
#End Region

#Region "Color, Depth & Skeleton Tracking"
        Protected Overrides Sub GetSkeletonTracked()
            If Me.SkeletalTrackingReferenceArea IsNot Nothing Then
                For Each s In Skeletons
                    If s.TrackingState = SkeletonTrackingState.Tracked Then
                        Dim skeletonHead = s.Joints(JointType.Head)
                        If skeletonHead.TrackingState <> JointTrackingState.NotTracked Then
                            Dim head = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(skeletonHead.Position, Sensor.ColorStream.Format)
                            If Me.SkeletalTrackingReferenceArea.IsInArea(head, skeletonHead.Position.Z, Sensor.ColorStream.FrameWidth, Sensor.ColorStream.FrameHeight) Then
                                Me.CurrentSkeleton = s
                                Exit For
                            End If
                        End If
                    End If
                Next
                SetVideoJointsPosition()
            End If
        End Sub

        Private Sub SetVideoJointsPosition()
            Try
                If CurrentSkeleton IsNot Nothing Then
                    If CurrentSkeleton.Joints(JointType.Head).TrackingState <> JointTrackingState.NotTracked Then
                        Dim headPosition = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(CurrentSkeleton.Joints(JointType.Head).Position, Sensor.ColorStream.Format)
                        Me.HeadX = headPosition.X * ImageColorWidth / Sensor.ColorStream.FrameWidth
                        Me.HeadY = headPosition.Y * ImageColorHeight / Sensor.ColorStream.FrameHeight
                    Else
                        Me.HeadX = Double.NaN
                        Me.HeadY = Double.NaN
                    End If
                    If CurrentSkeleton.Joints(JointType.HandRight).TrackingState <> JointTrackingState.NotTracked Then
                        Dim rightHandPosition = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(CurrentSkeleton.Joints(JointType.HandRight).Position, Sensor.ColorStream.Format)
                        Me.HandRightX = rightHandPosition.X * ImageColorWidth / Sensor.ColorStream.FrameWidth
                        Me.HandRightY = rightHandPosition.Y * ImageColorHeight / Sensor.ColorStream.FrameHeight
                    Else
                        Me.HandRightX = Double.NaN
                        Me.HandRightY = Double.NaN
                    End If
                    If CurrentSkeleton.Joints(JointType.HandLeft).TrackingState <> JointTrackingState.NotTracked Then
                        Dim leftHandPosition = Sensor.CoordinateMapper.MapSkeletonPointToColorPoint(CurrentSkeleton.Joints(JointType.HandLeft).Position, Sensor.ColorStream.Format)
                        Me.HandLeftX = leftHandPosition.X * ImageColorWidth / Sensor.ColorStream.FrameWidth
                        Me.HandLeftY = leftHandPosition.Y * ImageColorHeight / Sensor.ColorStream.FrameHeight
                    Else
                        Me.HandLeftX = Double.NaN
                        Me.HandLeftY = Double.NaN
                    End If
                Else
                    Me.HeadX = Double.NaN
                    Me.HeadY = Double.NaN
                    Me.HandRightX = Double.NaN
                    Me.HandRightY = Double.NaN
                    Me.HandLeftX = Double.NaN
                    Me.HandLeftY = Double.NaN
                End If
            Catch ex As Exception
                Me.HeadX = Double.NaN
                Me.HeadY = Double.NaN
                Me.HandRightX = Double.NaN
                Me.HandRightY = Double.NaN
                Me.HandLeftX = Double.NaN
                Me.HandLeftY = Double.NaN
            End Try
        End Sub
#End Region

#End Region

#Region "RobotControl"
        Private Function CanCommandSend() As Boolean
            Return True
        End Function

        Public Sub [Stop]()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.Stop()
            End If
        End Sub

        Public Sub Forward()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveForeward()
            End If
        End Sub

        Public Sub Backward()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveBackward()
            End If
        End Sub

        Public Sub TurnLeft()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.TurnLeft()
            End If
        End Sub

        Public Sub TurnRight()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.TurnRight()
            End If
        End Sub

        Public Sub ForwardLeft()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveForewardLeft()
            End If
        End Sub

        Public Sub ForwardRight()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveForewardRight()
            End If
        End Sub

        Public Sub BackwardLeft()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveBackwardLeft()
            End If
        End Sub

        Public Sub BackwardRight()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MoveBackwardRight()
            End If
        End Sub

        Public Sub MotorOff()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MotorOff()
            End If
        End Sub

        Public Sub MotorOn()
            If RobotControl IsNot Nothing AndAlso CanCommandSend() Then
                RobotControl.MotorOn()
            End If
        End Sub
#End Region

#Region "CameraControl"
        Public Sub CameraOn()
            If CameraControl IsNot Nothing AndAlso Not IsCameraOn Then
                CameraControl.Connect(CameraAddress, CameraPassword)
                IsCameraOn = True
            End If
        End Sub

        Public Sub CameraOff()
            If CameraControl IsNot Nothing AndAlso IsCameraOn Then
                CameraControl.Disconnect()
                IsCameraOn = False
            End If
        End Sub

        Public Sub CameraRotate(cw As Boolean)
            If CameraControl IsNot Nothing AndAlso IsCameraOn Then
                If cw Then
                    CameraControl.Move(GadgeteerManager.CameraDirection.Right, 5)
                Else
                    CameraControl.Move(GadgeteerManager.CameraDirection.Left, 5)
                End If
            End If
        End Sub
#End Region


#Region "Macchine a stati"
        Private Shared Sub CreateRobotStateMachine()
            StateMachineController.CreateMachine(Of GadgeteerWindowViewModel, RobotState, RobotGesture)("RobotState") _
                .AddTransition(RobotState.Stop, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.Stop, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.Stop, RobotGesture.Right, RobotState.TurnRight, Sub(a) a.TurnRight()) _
                .AddTransition(RobotState.Stop, RobotGesture.Left, RobotState.TurnLeft, Sub(a) a.TurnLeft()) _
                .AddTransition(RobotState.Stop, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.Stop, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.Stop, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.Stop, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.Forward, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.Forward, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.Forward, RobotGesture.Right, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.Forward, RobotGesture.Left, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.Forward, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.Forward, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.Forward, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.Forward, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.Backward, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.Backward, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.Backward, RobotGesture.Right, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.Backward, RobotGesture.Left, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.Backward, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.Backward, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.Backward, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.Backward, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.Forward, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.Backward, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.Left, RobotState.TurnLeft, Sub(a) a.TurnLeft()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.TurnRight, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.Forward, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.Backward, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.Right, RobotState.TurnRight, Sub(a) a.TurnRight()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.TurnLeft, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.Left, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.ForwardRight, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.Right, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.ForwardLeft, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.Left, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.BackwardRight, RobotState.BackwardRight) _
                .AddTransition(RobotState.BackwardRight, RobotGesture.BackwardLeft, RobotState.BackwardLeft, Sub(a) a.BackwardLeft()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.None, RobotState.Stop, Sub(a) a.Stop()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.Forward, RobotState.Forward, Sub(a) a.Forward()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.Backward, RobotState.Backward, Sub(a) a.Backward()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.Right, RobotState.BackwardRight, Sub(a) a.BackwardRight()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.ForwardRight, RobotState.ForwardRight, Sub(a) a.ForwardRight()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.ForwardLeft, RobotState.ForwardLeft, Sub(a) a.ForwardLeft()) _
                .AddTransition(RobotState.BackwardLeft, RobotGesture.BackwardRight, RobotState.BackwardRight, Sub(a) a.BackwardRight())

        End Sub
#End Region
    End Class
End Namespace