﻿
Imports System.ComponentModel
Imports Microsoft.Practices.Unity

Namespace ViewModels
    Public MustInherit Class ViewModelBase
        Implements INotifyPropertyChanged, IDisposable

        Protected ReadOnly Property IsInDesignMode As Boolean
            Get
                Return DesignerProperties.GetIsInDesignMode(New DependencyObject())
            End Get
        End Property

        Public Sub New(container As IUnityContainer)
            If container Is Nothing Then Throw New ArgumentNullException
            _Container = container
        End Sub

        Private _Container As IUnityContainer
        Protected ReadOnly Property Container As IUnityContainer
            Get
                Return _Container
            End Get
        End Property

#Region "INotifyPropertyChanged"
        Protected Sub OnPropertyChanged(propertyName As String)
            RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(propertyName))
        End Sub

        Public Event PropertyChanged(sender As Object, e As PropertyChangedEventArgs) Implements INotifyPropertyChanged.PropertyChanged
#End Region

#Region "IDisposable Support"
        Private disposedValue As Boolean ' To detect redundant calls

        ' IDisposable
        Protected Overridable Sub Dispose(disposing As Boolean)
            If Not Me.disposedValue Then
                If disposing Then
                    ' TODO: dispose managed state (managed objects).
                End If

                ' TODO: free unmanaged resources (unmanaged objects) and override Finalize() below.
                ' TODO: set large fields to null.
            End If
            Me.disposedValue = True
        End Sub

        ' TODO: override Finalize() only if Dispose(ByVal disposing As Boolean) above has code to free unmanaged resources.
        'Protected Overrides Sub Finalize()
        '    ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
        '    Dispose(False)
        '    MyBase.Finalize()
        'End Sub

        ' This code added by Visual Basic to correctly implement the disposable pattern.
        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(disposing As Boolean) above.
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub
#End Region

        Protected Sub NotifyAllPropertyChanged()
            OnPropertyChanged(Nothing)
        End Sub

    End Class
End Namespace