﻿
Imports KinectManager.Core.Gestures
Imports Microsoft.Kinect
Imports KinectManager.Core.Handlers
Imports KinectManager.Core.EventArgs

Public Class JointPositionChangeAreaGestureHandler
    Inherits GestureHandlerBase

    Public Sub New()

    End Sub

    Public Sub New(jointType As JointType)
        Me.JointType = jointType
    End Sub

    Public Property JointType As JointType = JointType.HipCenter

    Protected Property IsClicked As Boolean = False

    Protected Property LastAreaName As String = Nothing

    ''' <summary>
    ''' Gets or sets the click threshold.
    ''' </summary>
    ''' <value>The click threshold in m.</value>
    Public Property ClickThreshold As Double = 0.3D

    ''' <summary>
    ''' Gets or sets the click movement timeout.
    ''' </summary>
    ''' <value>The click movement timeout in milliseconds.</value>
    Public Property ClickMovementTimeout As Integer = 500

    Public Overrides Sub Elaborate(ByRef stopElaborate As Boolean)
        Dim lastFrame = GetLastFrame()
        Dim trackedSkeletons As IEnumerable(Of Skeleton) = GetTrackedSkeletons()
        If trackedSkeletons IsNot Nothing AndAlso trackedSkeletons.Any() Then
            'For Each s In trackedSkeletons
            '    ElaborateGesture(s, lastFrame.TimeStamp)
            'Next
            ElaborateGesture(trackedSkeletons.First(), lastFrame.TimeStamp)
        End If
    End Sub

    Private Sub ElaborateGesture(skeleton As Skeleton, timestamp As DateTime)
        If skeleton IsNot Nothing Then
            Dim joint = skeleton.Joints(Me.JointType)
            If joint.TrackingState = JointTrackingState.Tracked Then
                CheckClick(skeleton, timestamp)
                Dim query = From a In Me.ReferenceAreas
                            Where a.IsInArea(joint.Position.X, joint.Position.Y, joint.Position.Z) _
                            And a.Name <> LastAreaName
                            Select a

                If query.Any() Then
                    LastAreaName = query.FirstOrDefault().Name
                    OnGestureRecognized(CreateGestureRecognizedEventArgs(skeleton, timestamp, joint))
                End If
            End If
        End If
    End Sub

    Private Function CreateGesture() As Gesture
        Dim gesture = New Gesture
        gesture.Name = Me.Name
        gesture.Data.Add("JointType", Me.JointType)
        gesture.Data.Add("IsClicked", IsClicked)
        Return gesture
    End Function

    Private Function CreateGestureRecognizedEventArgs(skeleton As Skeleton, timestamp As DateTime, joint As Joint) As GestureRecognizedEventArgs
        Dim arg = New GestureRecognizedEventArgs()
        arg.Gesture = CreateGesture()
        arg.ReferenceAreas = Me.ReferenceAreas
        arg.Skeleton = Skeleton
        arg.Joints = {Joint}
        arg.Timestamp = timestamp
        Return arg
    End Function

    Private Sub CheckClick(skeleton As Skeleton, timestamp As DateTime)
        If JointType <> Microsoft.Kinect.JointType.Head Then
            If skeleton IsNot Nothing AndAlso skeleton.TrackingState = SkeletonTrackingState.Tracked Then
                Dim head = skeleton.Joints(Microsoft.Kinect.JointType.Head)
                Dim currentJoint = skeleton.Joints(Me.JointType)
                If head.TrackingState = JointTrackingState.Tracked And currentJoint.TrackingState = JointTrackingState.Tracked Then
                    If head.Position.Z - currentJoint.Position.Z > ClickThreshold Then
                        IsClicked = True
                    Else
                        IsClicked = False
                    End If
                End If
            Else
                IsClicked = False
            End If
        Else
            IsClicked = False
        End If
    End Sub

    Public Overrides Property Name As String = Me.GetType().Name
End Class
