﻿Namespace Interfaces
    ''' <summary>
    ''' Interfaccia che definisce il comportamento di una macchina a stati visibile solamente all'interno del framework.
    ''' </summary>
    ''' <typeparam name="TEntity">Tipo dell'entità.</typeparam>
    ''' <typeparam name="TState">Tipo dello stato.</typeparam>
    ''' <typeparam name="TEvent">Tipo dell'evento.</typeparam>
    ''' <remarks>E' la versione tipizzata dell'interfaccia che racchiude i metodi che dipendono dai tipi delle entità, degli stati e degli eventi.</remarks>
    Friend Interface IInnerStateMachine(Of TEntity, TState, TEvent)
        Inherits IInnerStateMachine

        ''' <summary>
        ''' Esegue il cambiamento di stato per una entità a seguito di un evento e restituisce lo stato precedente.
        ''' </summary>
        ''' <param name="entity">Entità di cui eseguire il cambio di stato.</param>
        ''' <param name="event">Evento del cambiamento di stato.</param>
        ''' <param name="OldState">Valore precedente dello stato a seguito del cambiamento.</param>
        ''' <returns>Restituisce True se l'operazione è andata a buon fine, False altrimenti.</returns>
        ''' <remarks></remarks>
        Function ChangeState(ByVal entity As TEntity, ByVal [event] As TEvent, ByRef OldState As TState) As Boolean
    End Interface

    ''' <summary>
    ''' Interfaccia che definisce il comportamento di una macchina a stati visibile solamente all'interno del framework.
    ''' </summary>
    ''' <remarks>E' la definizione non tipizzata dell'interfaccia che racchiude i metodi che non dipendono dai tipi delle entità, degli stati e degli eventi.</remarks>
    Friend Interface IInnerStateMachine

        ''' <summary>
        ''' Imposta la proprietà di stato di una macchina a stati.
        ''' </summary>
        ''' <param name="propertyName">Nome della proprietà.</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function SetStateProperty(ByVal propertyName As String) As Boolean

        ''' <summary>
        ''' Ripulisce la macchina a stati.
        ''' </summary>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function Clear() As Boolean
    End Interface
End Namespace