﻿Namespace Interfaces
    ''' <summary>
    ''' Interfaccia che descrive le funzionalità della macchina a stati esposte all'esterno del framework.
    ''' </summary>
    ''' <typeparam name="TEntity">Tipo dell'entità.</typeparam>
    ''' <typeparam name="TState">Tipo dello stato.</typeparam>
    ''' <typeparam name="TEvent">Tipo dell'evento.</typeparam>
    ''' <remarks></remarks>
    Public Interface IOuterStateMachine(Of TEntity, TState, TEvent)

        ''' <summary>
        ''' Aggiunge una transizione senza funzionalità di controllo alla macchina a stati.
        ''' </summary>
        ''' <param name="startState">Stato iniziale della transizione.</param>
        ''' <param name="event">Evento di transizione.</param>
        ''' <param name="endState">Stato finale della transizione.</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function AddTransition(ByVal startState As TState, _
                               ByVal [event] As TEvent, _
                               ByVal endState As TState) As IOuterStateMachine(Of TEntity, TState, TEvent)

        ''' <summary>
        ''' Aggiunge una transizione con funzionalità di controllo alla macchina a stati.
        ''' </summary>
        ''' <param name="startState">Stato iniziale della transizione.</param>
        ''' <param name="event">Evento di transizione.</param>
        ''' <param name="endState">Stato finale della transizione.</param>
        ''' <param name="conditionalLamdba">Lambda expression di controllo. La funzione viene richiamata prima del cambio di stato e se restituisce true, il cambiamento viene eseguito.</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function AddTransition(ByVal startState As TState, _
                               ByVal [event] As TEvent, _
                               ByVal endState As TState, _
                               ByVal conditionalLamdba As Func(Of TEntity, Boolean)) As IOuterStateMachine(Of TEntity, TState, TEvent)

        ''' <summary>
        ''' Aggiunge una transizione con funzionalità di controllo alla macchina a stati.
        ''' </summary>
        ''' <param name="startState">Stato iniziale della transizione.</param>
        ''' <param name="event">Evento di transizione.</param>
        ''' <param name="endState">Stato finale della transizione.</param>
        ''' <param name="executeLamdba">Lambda expression eseguita in caso di cambio stato. La sub viene richiamata al cambio di stato.</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function AddTransition(ByVal startState As TState, _
                                      ByVal [event] As TEvent, _
                                      ByVal endState As TState, _
                                      ByVal executeLamdba As Action(Of TEntity)) As IOuterStateMachine(Of TEntity, TState, TEvent)

        ''' <summary>
        ''' Aggiunge una transizione con funzionalità di controllo alla macchina a stati.
        ''' </summary>
        ''' <param name="startState">Stato iniziale della transizione.</param>
        ''' <param name="event">Evento di transizione.</param>
        ''' <param name="endState">Stato finale della transizione.</param>
        ''' <param name="conditionalLamdba">Lambda expression di controllo. La funzione viene richiamata prima del cambio di stato e se restituisce true, il cambiamento viene eseguito.</param>
        ''' <param name="executeLamdba">Lambda expression eseguita in caso di cambio stato. La sub viene richiamata al cambio di stato.</param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Function AddTransition(ByVal startState As TState, _
                                  ByVal [event] As TEvent, _
                                  ByVal endState As TState, _
                                  ByVal conditionalLamdba As Func(Of TEntity, Boolean),
                                  ByVal executeLamdba As Action(Of TEntity)) As IOuterStateMachine(Of TEntity, TState, TEvent)
    End Interface
End Namespace
