﻿Namespace Interfaces
    ''' <summary>
    ''' Definisce l'interfaccia di un oggetto elegibile come observer per il controller delle macchine a stati
    ''' </summary>
    ''' <remarks></remarks>
    Public Interface IStateMachineObserver

        ''' <summary>
        ''' Metodo richiamato nel momento in cui il framework di workflow si appresta a cambiare stato ad una entità.
        ''' </summary>
        ''' <param name="entity">Istanza dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="entityType">Tipo dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="event">Istanza dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="eventType">Tipo dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="stateType">Tipo dello stato.</param>
        ''' <param name="cancel">Argomento che abilita alla cancellazione dell'operazione di cambiamento di stato. Se impostato a True, l'operazione viene terminata e non viene eseguito il cambio di stato.</param>
        ''' <remarks></remarks>
        Sub StateChanging(ByVal entity As Object, _
                          ByVal entityType As Type, _
                          ByVal [event] As Object, _
                          ByVal eventType As Type, _
                          ByVal stateType As Type, _
                          ByRef cancel As Boolean)

        ''' <summary>
        ''' Metodo richiamato subito dopo che il framework ha eseguito un cambio di stato su un'entità.
        ''' </summary>
        ''' <param name="entity">Istanza dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="entityType">Tipo dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="event">Istanza dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="eventType">Tipo dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="oldState">Istanza dello stato prima del cambiamento.</param>
        ''' <param name="stateType">Tipo dello stato.</param>
        ''' <remarks></remarks>
        Sub StateChanged(ByVal entity As Object, _
                         ByVal entityType As Type, _
                         ByVal [event] As Object, _
                         ByVal eventType As Type, _
                         ByVal oldState As Object, _
                         ByVal stateType As Type)

        ''' <summary>
        ''' Metodo richiamato nel momento in cui si verifica un errore nell'operazione di cambio di stato.
        ''' </summary>
        ''' <param name="entity">Istanza dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="entityType">Tipo dell'entità sottoposta al cambio di stato.</param>
        ''' <param name="event">Istanza dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="eventType">Tipo dell'evento che ha generato il cambio di stato.</param>
        ''' <param name="stateType">Tipo dello stato.</param>
        ''' <remarks></remarks>
        Sub StateChangeError(ByVal entity As Object, _
                             ByVal entityType As Type, _
                             ByVal [event] As Object, _
                             ByVal eventType As Type, _
                             ByVal stateType As Type)

    End Interface

End Namespace