﻿Namespace Internals
    ''' <summary>
    ''' Classe contenente i dati di transizione (in termini di stato di arrivo e lambda di controllo) associati ad una coppia Stato Partenza, Evento.
    ''' </summary>
    ''' <typeparam name="TEntity">Tipo dell'entità coinvolta nella transizione.</typeparam>
    ''' <typeparam name="TState">Tipo dello stato della transizione.</typeparam>
    ''' <remarks></remarks>
    Friend Class TransitionData(Of TEntity, TState)

        Private _State As TState

        ''' <summary>
        ''' Stato di arrivo della transizione.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property State() As TState
            Get
                Return Me._State
            End Get
            Set(ByVal value As TState)
                Me._State = value
            End Set
        End Property


        Private _ConditionLambda As Func(Of TEntity, Boolean)

        ''' <summary>
        ''' Eventuale lambda expression di controllo.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property ConditionLambda() As Func(Of TEntity, Boolean)
            Get
                Return Me._ConditionLambda
            End Get
            Set(ByVal value As Func(Of TEntity, Boolean))
                Me._ConditionLambda = value
            End Set
        End Property

        Private _ExecuteLambda As Action(Of TEntity)

        ''' <summary>
        ''' Eventuale Action da eseguire al cambio di stato.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property ExecuteLambda() As Action(Of TEntity)
            Get
                Return Me._ExecuteLambda
            End Get
            Set(ByVal value As Action(Of TEntity))
                Me._ExecuteLambda = value
            End Set
        End Property

    End Class
End Namespace