﻿
Imports StateMachineManager.Utilities

Namespace Internals
    ''' <summary>
    ''' Classe che memorizza la coppia Stato,Evento per identificare una transizione.
    ''' </summary>
    ''' <typeparam name="TState">Tipo dello stato della transizione.</typeparam>
    ''' <typeparam name="TEvent">Tipo dell'evento della transizione.</typeparam>
    ''' <remarks></remarks>
    Friend Class TransitionKey(Of TState, TEvent)
        Implements IEquatable(Of TransitionKey(Of TState, TEvent)), IEqualityComparer(Of TransitionKey(Of TState, TEvent))

        Private _State As TState

        ''' <summary>
        ''' Stato di partenza.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property State() As TState
            Get
                Return Me._State
            End Get
            Set(ByVal value As TState)
                Me._State = value
            End Set
        End Property

        Private _Event As TEvent

        ''' <summary>
        ''' Evento di transizione.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property [Event]() As TEvent
            Get
                Return Me._Event
            End Get
            Set(ByVal value As TEvent)
                Me._Event = value
            End Set
        End Property

        Public Function Equals1(ByVal other As TransitionKey(Of TState, TEvent)) As Boolean Implements System.IEquatable(Of TransitionKey(Of TState, TEvent)).Equals
            Return Me.Equals(other)
        End Function

        Public Overrides Function Equals(ByVal obj As Object) As Boolean
            If TypeOf obj Is TransitionKey(Of TState, TEvent) Then
                Return Me = CType(obj, TransitionKey(Of TState, TEvent))
            Else
                Throw New ArgumentException()
            End If
        End Function

        Public Overrides Function GetHashCode() As Integer
            Return HashUtility.RSHash(Me.State, Me.Event)
        End Function

        Public Shared Operator <>(ByVal left As TransitionKey(Of TState, TEvent), ByVal right As TransitionKey(Of TState, TEvent)) As Boolean
            Return Not (right = left)
        End Operator

        Public Shared Operator =(ByVal left As TransitionKey(Of TState, TEvent), ByVal right As TransitionKey(Of TState, TEvent)) As Boolean
            Return (right.Event.Equals(left.Event) And right.State.Equals(left.State))
        End Operator

        Public Function Equals2(ByVal x As TransitionKey(Of TState, TEvent), ByVal y As TransitionKey(Of TState, TEvent)) As Boolean Implements System.Collections.Generic.IEqualityComparer(Of TransitionKey(Of TState, TEvent)).Equals
            Return x = y
        End Function

        Public Function GetHashCode1(ByVal obj As TransitionKey(Of TState, TEvent)) As Integer Implements System.Collections.Generic.IEqualityComparer(Of TransitionKey(Of TState, TEvent)).GetHashCode
            Return Me.GetHashCode
        End Function
    End Class
End Namespace